#include "os.h"

// IDT entry
struct idt_entry
{
    USHORT base_lo;
    USHORT sel;
    UCHAR always0;
    UCHAR flags;
    USHORT base_hi;
}__attribute__((packed)); //prevent compiler optimization

struct idt_ptr
{
    USHORT limit;
    ULONG  base;
}__attribute__((packed)); //prevent compiler optimization

// Declare an IDT of 256 entries and a pointer to the IDT
struct idt_entry idt[256];
struct idt_ptr   idt_register;

static void idt_load(){ asm volatile("lidt %0" : "=m" (idt_register)); } // load IDT register (IDTR)

// Put an entry into the IDT
void idt_set_gate(UCHAR num, ULONG base, USHORT sel, UCHAR flags)
{
    idt[num].base_lo = (base        & 0xFFFF);
    idt[num].base_hi = (base >> 16) & 0xFFFF;
    idt[num].sel     =   sel;
    idt[num].always0 =     0;
    idt[num].flags   = flags;
}

void idt_install()
{
    // Sets the special IDT pointer up
    idt_register.limit = (sizeof (struct idt_entry) * 256)-1;
    idt_register.base  = (ULONG) &idt;

    k_memset(&idt, 0, sizeof(struct idt_entry) * 256); // Clear out the entire IDT

    // Add any new ISRs to the IDT here using idt_set_gate
    // ...

    idt_load(); // The IDT register (IDTR) has to point to the IDT
}
